/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Roberto Fasciolo      (c) 2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Roberto Fasciolo 'robyf'     ** |  /========\  | */
/* |/   ======   \| **     e-mail:  robyf@libero.it     ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: netip.c,v 1.7 2001/08/21 07:15:58 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: netip.c,v 1.7 2001/08/21 07:15:58 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataNetIp.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <netinet/in.h>
#include <net/if.h>
#include <ctype.h>

#define MOD_MAX_COUNT   (10)

#define MOD_NET_SIZE    (42)
#define MOD_NET_SIZE_H  (16)                     /* Must be 16 !! */

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int            iCount;
   size_t         iBuffSize;
   char *         pBuffer;
   XpmIcon        xpmBGrnd;
   char           achNet[16];
   int            iIp[4];
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;
   const char * pNet = "lo";

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options] [device]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n"
                  "device:\n"
                  "   lo eth1 tr0  : device name\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.7 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else if ((argv[i][0] != '-') && (argv[i][0] != '+'))
         {
         pNet = argv[i];
         if (strlen (pNet) > 14)
            {
            fprintf (stderr, "%s: parameter %s too long\n", argv[0], argv[i]);
            iRc = 1;
            }
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount      = MOD_MAX_COUNT;
   pModData->iBuffSize   = 2048;
   pModData->pBuffer     = (char *) malloc (pModData->iBuffSize);
   strcpy (pModData->achNet, pNet);
   strcat (pModData->achNet, ":");
   for (i=0; i<4; i++)
      pModData->iIp[i] = 256;

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataNetIp_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;
   int i;
   char pNet[16];
   int hSocket;
   struct ifreq ifr;


   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   strcpy(pNet, pModData->achNet);
   pNet[strlen(pNet)-1] = 0;

   hSocket = socket (AF_INET, SOCK_DGRAM, 0);
   if (hSocket == -1)
      {
      return(0);
      }

   strcpy(ifr.ifr_name, pNet);
   if (ioctl (hSocket, SIOCGIFFLAGS, &ifr) < 0)
      {
      close (hSocket);
      return (0);
      }

   if (ifr.ifr_flags & IFF_UP)
      {
      strcpy(ifr.ifr_name, pNet);
      if (ioctl (hSocket, SIOCGIFADDR, &ifr) < 0)
         {
         close (hSocket);
         return (0);
         }
      else
         {
         for(i=0;i<4;i++)
            pModData->iIp[i] = (unsigned char)ifr.ifr_dstaddr.sa_data[i+2];
         }
      }
   else
      {
      for(i=0;i<4;i++)
         pModData->iIp[i] = 256;
      }

   close (hSocket);

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   int i;
   char achBuffer[4];

   iRc =  XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                     0, 0,
                     pModData->xpmBGrnd.attributes.width,
                     pModData->xpmBGrnd.attributes.height,
                     iX, iY);

   i = strlen (pModData->achNet);
   pModData->achNet[i-1] = 0;
   DrawLetterString (pModData->achNet, pDisplay, hWindow, hGC,
                     iX + 46 - ((i - 1) * 6), iY + 2);
   pModData->achNet[i-1] = ':';


   if (pModData->iIp[0] == 256)
      strcpy (achBuffer, "   ");
   else
      sprintf (achBuffer, "%3d", pModData->iIp[0]);

   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 9, iY + 12, 0);

   if (pModData->iIp[1] == 256)
      strcpy (achBuffer, "   ");
   else
      sprintf (achBuffer, "%3d", pModData->iIp[1]);

   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 31 , iY + 12, 0);

   if (pModData->iIp[2] == 256)
      strcpy (achBuffer, "   ");
   else
      sprintf (achBuffer, "%3d", pModData->iIp[2]);

   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 9, iY + 22, 0);

   if (pModData->iIp[3] == 256)
      strcpy (achBuffer, "   ");
   else
      sprintf (achBuffer, "%3d", pModData->iIp[3]);

   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 31 , iY + 22, 0);

   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.7 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      if (pModData->iIp[0] != 256)
         {
         snprintf (pModData->pBuffer,
                   pModData->iBuffSize,
                   " netip  V %s\n"
                   "\n"
                   "Ip %s %d.%d.%d.%d",
                   achRev + 11,
                   pModData->achNet,
                   pModData->iIp[0], pModData->iIp[1],
                   pModData->iIp[2], pModData->iIp[3]);
         }
      else
         {
         snprintf (pModData->pBuffer,
                   pModData->iBuffSize,
                   " netip  V %s\n"
                   "\n"
                   "Ip %s None set",
                   achRev + 11,
                   pModData->achNet);
         }
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

