/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Roberto Fasciolo      (c) 2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Roberto Fasciolo 'robyf'     ** |  /========\  | */
/* |/   ======   \| **     e-mail:  robyf@libero.it     ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: pcmcia.c,v 1.12 2003/11/19 15:11:13 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: pcmcia.c,v 1.12 2003/11/19 15:11:13 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataPcmcia.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <errno.h>
#include <fcntl.h>
#include <sys/ioctl.h>

#define MOD_MAX_COUNT   (10)

/****************************************************************************/
/* Pcmcia Types, as in pcmcia/cs_types.h                                    */
/****************************************************************************/

typedef u_int   event_t;

/****************************************************************************/
/* Pcmcia Types, as in pcmcia/cs.h                                          */
/****************************************************************************/

typedef struct cs_status_t {
    u_char      Function;
    event_t     CardState;
    event_t     SocketState;
} cs_status_t;

/****************************************************************************/
/* Pcmcia Types, as in pcmcia/ds.h                                          */
/****************************************************************************/

//#define DS_GET_STATUS                   _IOWR('d', 9, cs_status_t)
#define DS_GET_STATUS                   _IOWR_BAD('d', 9, cs_status_t)

/****************************************************************************/
/* Pcmcia Types, as in pcmcia/cs.h                                          */
/****************************************************************************/

#define CS_EVENT_PRI_LOW                0
#define CS_EVENT_PRI_HIGH               1

#define CS_EVENT_WRITE_PROTECT          0x000001
#define CS_EVENT_CARD_LOCK              0x000002
#define CS_EVENT_CARD_INSERTION         0x000004
#define CS_EVENT_CARD_REMOVAL           0x000008
#define CS_EVENT_BATTERY_DEAD           0x000010
#define CS_EVENT_BATTERY_LOW            0x000020
#define CS_EVENT_READY_CHANGE           0x000040
#define CS_EVENT_CARD_DETECT            0x000080
#define CS_EVENT_RESET_REQUEST          0x000100
#define CS_EVENT_RESET_PHYSICAL         0x000200
#define CS_EVENT_CARD_RESET             0x000400
#define CS_EVENT_REGISTRATION_COMPLETE  0x000800
#define CS_EVENT_RESET_COMPLETE         0x001000
#define CS_EVENT_PM_SUSPEND             0x002000
#define CS_EVENT_PM_RESUME              0x004000
#define CS_EVENT_INSERTION_REQUEST      0x008000
#define CS_EVENT_EJECTION_REQUEST       0x010000
#define CS_EVENT_MTD_REQUEST            0x020000
#define CS_EVENT_ERASE_COMPLETE         0x040000
#define CS_EVENT_REQUEST_ATTENTION      0x080000
#define CS_EVENT_CB_DETECT              0x100000
#define CS_EVENT_3VCARD                 0x200000
#define CS_EVENT_XVCARD                 0x400000

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int           iCount;
   size_t        iBuffSize;
   char *        pStabName;
   char *        pBuffer;
   int           iMajor;
   int           iSockNbr;
   int           iInSock;
   int           iIsSusp;
   int           iIsCardBus;
   int           iIsPcCard;
   int           iReqAttn;
   int           iBusy;
   int           iBattery;
   char          achPcmcia[3];
   XpmIcon       xpmBGrnd;
   Pixel         pxRed;
   Pixel         pxGreen;
   } ModuleData;

/****************************************************************************/
/* open_sock(), as in cardctl.c                                             */
/****************************************************************************/

static int open_sock(int major, int sock) {
    int fd;
    char fn[255];

    sprintf(fn, "/dev/msysm.%d.%d", major, sock);

    fd = open(fn, O_RDONLY);
    return fd;
} /* open_sock */

/****************************************************************************/
/* lookup_dev(), as in cardctl.c                                            */
/****************************************************************************/

static int lookup_dev(const char *name)
{
    FILE *f;
    int n;
    char s[32], t[32];

    f = fopen("/proc/devices", "r");
    if (f == NULL)
        return -errno;
    while (fgets(s, 32, f) != NULL) {
        if (sscanf(s, "%d %s", &n, t) == 2)
            if (strcmp(name, t) == 0)
                break;
    }
    fclose(f);
    if (strcmp(name, t) == 0)
        return n;
    else
        return -ENODEV;
} /* lookup_dev */


/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;
   char chPcmcia = '0';
   char chPcmcia1 = 0 ;

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options] [0|1]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.12 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else if ((argv[i][0] >= '0') && (argv[i][0] <= '9') &&
               ((argv[i][1] == 0) ||
                ((argv[i][1] >= '0') && (argv[i][1] <= '9') &&
                 (argv[i][2] == 0))))
         {
         chPcmcia  = argv[i][0];
         chPcmcia1 = argv[i][1];
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount      = MOD_MAX_COUNT;
   pModData->iBuffSize   = 256;
   pModData->pBuffer     = (char *) malloc (pModData->iBuffSize);
   pModData->pStabName   = strdup("/var/state/pcmcia/stab");
   pModData->pxRed       = GetColor ("#FF0000", pDisplay, hWindow);
   pModData->pxGreen     = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->achPcmcia[0] = chPcmcia;
   pModData->achPcmcia[1] = chPcmcia1;
   pModData->achPcmcia[2] = 0;
   pModData->iSockNbr     = atoi(pModData->achPcmcia);
   pModData->iMajor       = lookup_dev("pcmcia");

   pModData->iInSock      = 0;
   pModData->iIsSusp      = 0;
   pModData->iIsCardBus   = 0;
   pModData->iIsPcCard    = 0;
   pModData->iReqAttn     = 0;
   pModData->iBusy        = 0;
   pModData->iBattery     = 0;

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataPcmcia_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData->pStabName);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   int iTmp;
   int iSock;
   ModuleData * pModData = (ModuleData *) pData;
   cs_status_t status;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   iSock = open_sock(pModData->iMajor, pModData->iSockNbr);

   for (iTmp=0; iTmp<4; iTmp++)
      {
      status.Function = iTmp;
      if (ioctl(iSock, DS_GET_STATUS, &status) == 0)
         {
         pModData->iInSock = 1;

         pModData->iIsCardBus = status.CardState & CS_EVENT_CB_DETECT;
         pModData->iIsPcCard  = status.CardState & CS_EVENT_CARD_DETECT;

         if (status.CardState & CS_EVENT_PM_SUSPEND)
            {
            pModData->iIsSusp  = 1;
            pModData->iReqAttn = 0;
            pModData->iBusy    = 0;
            pModData->iBattery = 0;
            }
         else
            {
            pModData->iIsSusp = 0;
            pModData->iReqAttn = status.CardState & CS_EVENT_REQUEST_ATTENTION;
            pModData->iBusy    = status.CardState & CS_EVENT_READY_CHANGE;

            if (status.CardState & CS_EVENT_BATTERY_DEAD)
               pModData->iBattery = 2;
            else if(status.CardState & CS_EVENT_BATTERY_LOW)
               pModData->iBattery = 1;
            else
               pModData->iBattery = 0;
            }
         }
      else
         {
         if (iTmp == 0)
            {
            if (errno == ENODEV)
               pModData->iInSock = 0;
            }
         break;
         }
      }


   close(iSock);

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   int iLen;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;

   iRc = XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                    0, 0,
                    pModData->xpmBGrnd.attributes.width,
                    pModData->xpmBGrnd.attributes.height,
                    iX, iY);

   DrawLetterString (pModData->achPcmcia, pDisplay, hWindow, hGC,
                     iX + 36, iY + 1);


   if (pModData->iInSock != 0)
      {
      gcVal.foreground = pModData->pxGreen;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      for (iLen = 0; iLen < 6; iLen++)
         {
         XDrawLine (pDisplay, hWindow, hGC, iX + 9 + iLen, iY + 14,
                    iX + 9 + iLen, iY + 16);
         }

      if (pModData->iIsSusp != 0)
         {
         gcVal.foreground = pModData->pxGreen;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         for (iLen = 0; iLen < 6; iLen++)
            {
            XDrawLine (pDisplay, hWindow, hGC, iX + 39 + iLen, iY + 14,
                       iX + 39 + iLen, iY + 16);
            }
         }

      if (pModData->iIsCardBus != 0)
         {
         gcVal.foreground = pModData->pxGreen;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         for (iLen = 0; iLen < 5; iLen++)
            {
            XDrawLine (pDisplay, hWindow, hGC, iX + 3 + iLen, iY + 26,
                       iX + 3 + iLen, iY + 28);
            }
         }

      if (pModData->iIsPcCard != 0)
         {
         gcVal.foreground = pModData->pxGreen;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         for (iLen = 0; iLen < 5; iLen++)
            {
            XDrawLine (pDisplay, hWindow, hGC, iX + 11 + iLen, iY + 26,
                       iX + 11 + iLen, iY + 28);
            }
         }

      if (pModData->iReqAttn != 0)
         {
         gcVal.foreground = pModData->pxRed;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         for (iLen = 0; iLen < 5; iLen++)
            {
            XDrawLine (pDisplay, hWindow, hGC, iX + 20 + iLen, iY + 26,
                       iX + 20 + iLen, iY + 28);
            }
         }

      if (pModData->iBusy != 0)
         {
         gcVal.foreground = pModData->pxGreen;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         for (iLen = 0; iLen < 5; iLen++)
            {
            XDrawLine (pDisplay, hWindow, hGC, iX + 30 + iLen, iY + 26,
                       iX + 30 + iLen, iY + 28);
            }
         }

      if (pModData->iBattery != 0)
         {
         gcVal.foreground = (pModData->iBattery == 1) ?
                             pModData->pxGreen : pModData->pxRed;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         for (iLen = 0; iLen < 5; iLen++)
            {
            XDrawLine (pDisplay, hWindow, hGC, iX + 39 + iLen, iY + 26,
                       iX + 39 + iLen, iY + 28);
            }
         }
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.12 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                " pcmcia  V %s\n"
                "\n"
                "Socket %d: %s",
                achRev + 11,
                pModData->iSockNbr,
                (pModData->iInSock) ? "Ready " : "Empty ");

      if (pModData->iIsSusp)
         {
         strcat (pModData->pBuffer, "\n Suspended");
         }

      if (pModData->iIsCardBus)
         {
         strcat (pModData->pBuffer, "\n Card Bus");
         }

      if (pModData->iIsPcCard)
         {
         strcat (pModData->pBuffer, "\n Pc Card");
         }

      if (pModData->iReqAttn)
         {
         strcat (pModData->pBuffer, "\n Request Attention");
         }

      if (pModData->iBusy)
         {
         strcat (pModData->pBuffer, "\n Busy");
         }

      if (pModData->iBattery == 2)
         {
         strcat (pModData->pBuffer, "\n Battery dead");
         }
      else if (pModData->iBattery == 1)
         {
         strcat (pModData->pBuffer, "\n Battery low");
         }
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

