/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **       Masar System Monitor       ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Roberto Fasciolo      (c) 2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Roberto Fasciolo 'robyf'     ** |  /========\  | */
/* |/   ======   \| **     e-mail:  robyf@libero.it     ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: ppptime.c,v 1.9 2003/06/13 14:36:48 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: ppptime.c,v 1.9 2003/06/13 14:36:48 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataPppTime.xpm"
#include "dataPppTimeOff.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#define MOD_MAX_COUNT   (5)

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int           iCount;
   char          achBuffer[256];
   time_t        timeNow;
   struct tm     tmNow;
   XpmIcon       xpmBGrndOn;
   XpmIcon       xpmBGrndOff;
   Pixel         pxSec;
   Pixel         pxMin;
   Pixel         pxHour;
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.9 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount       = MOD_MAX_COUNT;
   pModData->achBuffer[0] = 0;
   pModData->pxSec        = GetColor ("#FFFF00", pDisplay, hWindow);
   pModData->pxMin        = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->pxHour       = GetColor ("#FF0000", pDisplay, hWindow);

   pModData->timeNow       = 0;
   pModData->tmNow.tm_sec  = 0;
   pModData->tmNow.tm_min  = 0;
   pModData->tmNow.tm_hour = 0;

   memset (&pModData->xpmBGrndOn, 0, sizeof (pModData->xpmBGrndOn));
   pModData->xpmBGrndOn.attributes.valuemask = (XpmReturnPixels |
                                                XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataPppTime_xpm,
                            &pModData->xpmBGrndOn.pixmap,
                            &pModData->xpmBGrndOn.mask,
                            &pModData->xpmBGrndOn.attributes);

   memset (&pModData->xpmBGrndOff, 0, sizeof (pModData->xpmBGrndOff));
   pModData->xpmBGrndOff.attributes.valuemask = (XpmReturnPixels |
                                                 XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataPppTimeOff_xpm,
                            &pModData->xpmBGrndOff.pixmap,
                            &pModData->xpmBGrndOff.mask,
                            &pModData->xpmBGrndOff.attributes);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrndOn.pixmap);
   XFreePixmap (pDisplay, pModData->xpmBGrndOff.pixmap);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;
   struct stat statBuff;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;


   if (!stat ("/var/run/ppp0.pid", &statBuff))
      {
      pModData->timeNow       = time (NULL) - statBuff.st_mtime;
      pModData->tmNow.tm_sec  = pModData->timeNow % 60;
      pModData->tmNow.tm_min  = (pModData->timeNow / 60) % 60;
      pModData->tmNow.tm_hour = pModData->timeNow / 3600;
      }
   else
      {
      pModData->timeNow       = 0;
      pModData->tmNow.tm_sec  = 0;
      pModData->tmNow.tm_min  = 0;
      pModData->tmNow.tm_hour = 0;
      }

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;
   char achBuffer[40];
   static const int iHour [29][2] =
      {
         { 12,  6 }, { 13,  6 }, { 14,  6 }, { 15,  7 }, { 16,  8 },
         { 17,  9 }, { 17, 10 }, { 17, 11 }, { 17, 12 }, { 17, 13 },
         { 16, 14 }, { 15, 15 }, { 14, 16 }, { 13, 16 }, { 12, 16 },
         { 11, 16 }, { 10, 16 }, {  9, 15 }, {  8, 14 }, {  7, 13 },
         {  7, 12 }, {  7, 11 }, {  7, 10 }, {  7,  9 }, {  8,  8 },
         {  9,  7 }, { 10,  6 }, { 11,  6 }, { 12,  6 }
      };
   static const int iMin [53][2] =
      {
         { 12,  2 }, { 13,  2 }, { 14,  2 }, { 15,  3 }, { 16,  3 },
         { 17,  4 }, { 18,  4 }, { 19,  5 }, { 19,  6 }, { 20,  7 },
         { 20,  8 }, { 21,  9 }, { 21, 10 }, { 21, 11 }, { 21, 12 },
         { 21, 13 }, { 20, 14 }, { 20, 15 }, { 19, 16 }, { 19, 17 },
         { 18, 18 }, { 17, 18 }, { 16, 19 }, { 15, 19 }, { 14, 20 },
         { 13, 20 }, { 12, 20 }, { 11, 20 }, { 10, 20 }, {  9, 19 },
         {  8, 19 }, {  7, 18 }, {  6, 18 }, {  5, 17 }, {  5, 16 },
         {  4, 15 }, {  4, 14 }, {  3, 13 }, {  3, 12 }, {  3, 11 },
         {  3, 10 }, {  3,  9 }, {  4,  8 }, {  4,  7 }, {  5,  6 },
         {  5,  5 }, {  6,  4 }, {  7,  4 }, {  8,  3 }, {  9,  3 },
         { 10,  2 }, { 11,  2 }, { 12,  2 }
      };

   if (pModData->timeNow == 0)
      {
     iRc = XCopyArea (pDisplay, pModData->xpmBGrndOff.pixmap, hWindow, hGC,
                      0, 0,
                      pModData->xpmBGrndOff.attributes.width,
                      pModData->xpmBGrndOff.attributes.height,
                      iX, iY);
      }
   else
      {
     iRc = XCopyArea (pDisplay, pModData->xpmBGrndOn.pixmap, hWindow, hGC,
                      0, 0,
                      pModData->xpmBGrndOn.attributes.width,
                      pModData->xpmBGrndOn.attributes.height,
                      iX, iY);

      achBuffer[0] = pModData->tmNow.tm_hour / 10 + '0';
      achBuffer[1] = pModData->tmNow.tm_hour % 10 + '0';
      achBuffer[2] = 0;
      DrawLetterString (achBuffer, pDisplay, hWindow, hGC, iX + 2, iY + 22);
      achBuffer[0] = pModData->tmNow.tm_min / 10 + '0';
      achBuffer[1] = pModData->tmNow.tm_min % 10 + '0';
      achBuffer[2] = 0;
      DrawLetterString (achBuffer, pDisplay, hWindow, hGC, iX + 17, iY + 22);
      achBuffer[0] = pModData->tmNow.tm_sec / 10 + '0';
      achBuffer[1] = pModData->tmNow.tm_sec % 10 + '0';
      achBuffer[2] = 0;
      DrawLetterString (achBuffer, pDisplay, hWindow, hGC, iX + 32, iY + 22);

      gcVal.foreground = pModData->pxMin;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      XDrawLine (pDisplay, hWindow, hGC, iX + 35, iY + 11,
                 iX + iMin[pModData->tmNow.tm_min*52/60][0] + 23,
                 iY + iMin[pModData->tmNow.tm_min*52/60][1]);

      gcVal.foreground = pModData->pxHour;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      XDrawLine (pDisplay, hWindow, hGC, iX + 35, iY + 11,
                 iX + iHour[pModData->tmNow.tm_hour%12*28/12][0] + 23,
                 iY + iHour[pModData->tmNow.tm_hour%12*28/12][1]);

      gcVal.foreground = pModData->pxSec;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      XDrawLine (pDisplay, hWindow, hGC, iX + 35, iY + 11,
                 iX + iMin[pModData->tmNow.tm_sec*52/60][0] + 23,
                 iY + iMin[pModData->tmNow.tm_sec*52/60][1]);
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.9 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   snprintf (pModData->achBuffer,
             sizeof (pModData->achBuffer),
             " ppptime  V %s\n"
             "\n"
             "    %2.2i:%2.2i:%2.2i",
             achRev + 11,
             pModData->tmNow.tm_hour,
             pModData->tmNow.tm_min,
             pModData->tmNow.tm_sec);

   return (pModData->achBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

