/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **       Masar System Monitor       ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Kiri Wagstaff         (c) 2001  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **      Kiri Wagstaff 'wkiri'       ** |  /========\  | */
/* |/   ======   \| **  e-mail:  wkiri@cs.cornell.edu   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: seti.c,v 1.3 2001/08/21 08:27:38 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] = "$Id";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataSeti.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>

/* We don't need to update very often */
#define MOD_MAX_COUNT   (100)

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
     int           iCount;
     char          achBuffer[256];
     XpmIcon       xpmBGrnd;       /* Basic background for seti display */
     char          stateFile[256]; /* Current setiathome state */
     char          userFile[256];  /* Info about the user running it */
     double        progress;       /* Progress on the current work unit */
     long          nResults;       /* Total number of work units done */
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;
   char* base = NULL; /* Temp var to hold the name of the setiathome dir */

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options] [directory]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n"
                  "directory:\n"
                  "   e.g. ~/Seti  : where setiathome was run from\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.3 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else
         {
           /* Stat the files */
           struct stat statBuf;
           char filename[256];
           snprintf(filename, sizeof(filename), "%s/state.sah", argv[i]);
           if (stat(filename, &statBuf)) {
             fprintf(stderr, "%s: failed to find %s\n", argv[0], filename);
             iRc = 1;
           }
           snprintf(filename, sizeof(filename), "%s/user_info.sah", argv[i]);
           if (stat(filename, &statBuf)) {
             fprintf(stderr, "%s: failed to find %s\n", argv[0], filename);
             iRc = 1;
           }
           base = strdup(argv[i]);
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   if (!base) {
     /* didn't get a path specified */
     fprintf(stderr, "%s: No path specified to setiathome files\n", argv[0]);
     return (NULL);
   }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount       = MOD_MAX_COUNT;
   pModData->achBuffer[0] = 0;

   /* Set up the background */
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                                XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataSeti_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   /* Set the filenames */
   snprintf(pModData->stateFile, sizeof (pModData->stateFile),
            "%s/state.sah", base);
   snprintf(pModData->userFile, sizeof (pModData->userFile),
            "%s/user_info.sah", base);

   /* Default initial values */
   pModData->progress = 0;
   pModData->nResults = 0;

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;
   char buf[256];
   FILE* pipe; /* pipe for grabbing info from setiathome files */

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   /********* Read the files for the progress info ***********/
   /* If we're between work units, state.sah won't exist.
    * That's okay; we'll just wait until it appears (future update).
    * In the meantime, direct stderr to /dev/null, to avoid
    * pestering the user. */
   snprintf(buf, sizeof (buf),
            "grep prog %s 2> /dev/null", pModData->stateFile);
   if (!(pipe = popen(buf, "r")))
      {
        return (0);
      }
   else
      {
        /* If we've just started a new work unit, state.sah
         * may not have a 'prog' line.  That's okay; we'll just
         * wait until it appears (future update).
         * In the meantime, set the progress to 0. */
        double d = 1.234;
        if (fscanf(pipe, "prog=%lf", &d) != 1)
          pModData->progress = 0;
        else
          pModData->progress = (int)(d*1000+0.5)/10.0;
        pclose(pipe);
      }
   snprintf(buf, sizeof (buf), "grep nresults %s", pModData->userFile);
   if (!(pipe = popen(buf, "r")))
      {
        return (0);
      }
   else
      {
        fscanf(pipe, "nresults=%ld", &pModData->nResults);
        pclose(pipe);
      }

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   /*   XGCValues gcVal;*/
   char achBuffer[7];
   int lVal, iLen;

   iRc = XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                    0, 0,
                    pModData->xpmBGrnd.attributes.width,
                    pModData->xpmBGrnd.attributes.height,
                    iX, iY);

   /* construct the buffer of digits for percent done */
   iLen = 2;
   achBuffer[iLen--] = 0;
   lVal = (int)(pModData->progress);
   while (iLen >= 0)
     {
       achBuffer[iLen--] = (lVal % 10) + '0';
       lVal /= 10;
     }
   /* Walk back through, blanking out zeros */
   iLen = 0;
   while ((iLen < 3) && (achBuffer[iLen] == '0'))
     {
       achBuffer[iLen++] = ' ';
     }
   DrawDigitString(achBuffer, pDisplay, hWindow, hGC, iX + 21, iY + 12, 0);
   achBuffer[0] = (int)(pModData->progress*10)%10 + '0';
   achBuffer[1] = 0;
   DrawDigitString(achBuffer, pDisplay, hWindow, hGC, iX + 33, iY + 12, 0);

   /* construct the buffer of digits for number of results */
   iLen = sizeof(achBuffer);
   achBuffer[iLen--] = 0;
   lVal = pModData->nResults;
   while (iLen >= 0)
     {
       achBuffer[iLen--] = (lVal % 10) + '0';
       lVal /= 10;
     }
   /* Walk back through, blanking out zeros */
   iLen = 0;
   while ((iLen < (int)sizeof(achBuffer)) && (achBuffer[iLen] == '0'))
     {
       achBuffer[iLen++] = ' ';
     }
   DrawDigitString(achBuffer, pDisplay, hWindow, hGC, iX + 6, iY + 22, 0);

   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.3 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   snprintf (pModData->achBuffer,
             sizeof (pModData->achBuffer),
             " seti  V %s",
             achRev + 11);

   return (pModData->achBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

