/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: modules.c,v 1.10 2000/09/08 14:40:38 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: modules.c,v 1.10 2000/09/08 14:40:38 cvs Exp $";

/****************************************************************************/

#include "msysm.h"
#include <msysm/modules.h>

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <dlfcn.h>

/****************************************************************************/
/*                          LoadModule ()                                   */
/****************************************************************************/
int LoadModule (int argc, const char * argv[],
                const char * pszLibPath, int fVertical,
                Display * pDisplay, Window hWindow)
   {
   Module * pMod;
   Module * pModNew;
   size_t iSize;

   *Psz_RCS = *Psz_RCS;                 // To fake compiler warnings

   if ((argc == 0) ||
       (argv[0] == NULL) ||
       (pszLibPath == NULL))
      {
      fprintf (stderr, "LoadModule: invalid parametes\n");
      return 1;
      }

   iSize = strlen (argv[0]) + strlen (pszLibPath) + 5; // "/" + ".so" + '\0'

   pModNew = (Module *) malloc (sizeof (Module));
   if (pModNew == NULL)
      {
      fprintf (stderr, "LoadModule: out of memory for %s\n", argv[0]);
      return 2;
      }

   pModNew->pNext       = NULL;
   pModNew->pcszModName = (char *) malloc (iSize);
   pModNew->iX          = 0;
   pModNew->iY          = 0;
   pModNew->uiWidth     = 48;
   pModNew->uiHeight    = 32;
   pModNew->pModData    = NULL;

   if (pModNew->pcszModName == NULL)
      {
      fprintf (stderr, "LoadModule: out of memory for %s\n", argv[0]);
      free (pModNew);
      return 3;
      }
   strcpy (pModNew->pcszModName, pszLibPath);
   strcat (pModNew->pcszModName, "/");
   strcat (pModNew->pcszModName, argv[0]);
   strcat (pModNew->pcszModName, ".so");

   pModNew->hDlModule   = dlopen (pModNew->pcszModName, RTLD_NOW);
   if (pModNew->hDlModule == NULL)
      {
      fprintf (stderr, "LoadModule: cannot load %s\n", pModNew->pcszModName);
      free (pModNew->pcszModName);
      free (pModNew);
      return 4;
      }

   pModNew->pfnLoad   = (PFN_LOAD)   dlsym (pModNew->hDlModule, "Load");
   pModNew->pfnUnLoad = (PFN_UNLOAD) dlsym (pModNew->hDlModule, "UnLoad");
   pModNew->pfnUpdate = (PFN_UPDATE) dlsym (pModNew->hDlModule, "Update");
   pModNew->pfnPaint  = (PFN_PAINT)  dlsym (pModNew->hDlModule, "Paint");
   pModNew->pfnInfo   = (PFN_INFO)   dlsym (pModNew->hDlModule, "Info");
   pModNew->pfnExec   = (PFN_EXEC)   dlsym (pModNew->hDlModule, "Exec");

   if ((pModNew->pfnLoad   == NULL) ||
       (pModNew->pfnUnLoad == NULL) ||
       (pModNew->pfnUpdate == NULL) ||
       (pModNew->pfnPaint  == NULL) ||
       (pModNew->pfnInfo   == NULL) ||
       (pModNew->pfnExec   == NULL))
      {
      fprintf (stderr, "LoadModule: cannot load functions of %s\n",
               pModNew->pcszModName);
      free (pModNew->pcszModName);
      free (pModNew);
      return 5;
      }

   pModNew->pModData = pModNew->pfnLoad (argc, argv, pDisplay, hWindow);
   if (pModNew->pModData == NULL)
      {
      free (pModNew->pcszModName);
      free (pModNew);
      return 6;
      }

   if (p_FirstModule == NULL)
      {
      p_FirstModule = pModNew;
      return 0;
      }

   pMod = p_FirstModule;
   while (pMod->pNext != NULL)
      {
      pMod = pMod->pNext;
      }
   pMod->pNext = pModNew;

   if (fVertical)
      {
      pModNew->iX = 0;
      pModNew->iY = pMod->iY + pMod->uiHeight;
      }
   else
      {
      pModNew->iX = pMod->iX + pMod->uiWidth;
      pModNew->iY = pMod->iY;
      }

   return 0;
   }

/****************************************************************************/
/*                          UnLoadAllModules ()                             */
/****************************************************************************/
void UnLoadAllModules (Display * pDisplay, Window hWindow)
   {
   Module * pMod;
   Module * pModNext;

   pMod = p_FirstModule;
   while (pMod != NULL)
      {
      pMod->pfnUnLoad (pMod->pModData, pDisplay, hWindow);

      dlclose (pMod->hDlModule);

      pModNext = pMod->pNext;
      free (pMod);
      pMod = pModNext;
      }

   p_FirstModule = NULL;
   }

