/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: apm.c,v 1.40 2003/06/13 14:36:48 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: apm.c,v 1.40 2003/06/13 14:36:48 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataApm.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define MOD_MAX_COUNT   (10)

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int           iCount;
   size_t        iBuffSize;
   char *        pBuffer;
   XpmIcon       xpmBGrnd;
   unsigned long ulFlags;
   long          lPercent;
   long          lMin;
   Pixel         pxRed;
   Pixel         pxGreen;
   Pixel         pxBlue[6];
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.40 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount      = MOD_MAX_COUNT;
   pModData->iBuffSize   = 256;
   pModData->pBuffer     = (char *) malloc (pModData->iBuffSize);
   pModData->ulFlags     = 0;
   pModData->lPercent    = 0;
   pModData->lMin        = 0;
   pModData->pxRed       = GetColor ("#FF0000", pDisplay, hWindow);
   pModData->pxGreen     = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->pxBlue[0]   = GetColor ("#3838F0", pDisplay, hWindow);
   pModData->pxBlue[1]   = GetColor ("#3434E0", pDisplay, hWindow);
   pModData->pxBlue[2]   = GetColor ("#3030D0", pDisplay, hWindow);
   pModData->pxBlue[3]   = GetColor ("#2828C0", pDisplay, hWindow);
   pModData->pxBlue[4]   = GetColor ("#2424B0", pDisplay, hWindow);
   pModData->pxBlue[5]   = GetColor ("#2020A0", pDisplay, hWindow);

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataApm_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   const char * pPtr;
   size_t iLen;
   int hMemFile;
   ModuleData * pModData = (ModuleData *) pData;
   int iBase;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   hMemFile = open ("/proc/apm", O_RDONLY, 0);
   if (hMemFile == -1)
      {
      return (0);
      }

   iLen = read (hMemFile, pModData->pBuffer, pModData->iBuffSize);
   close (hMemFile);

   if (iLen == pModData->iBuffSize)
      {
      pModData->iBuffSize *= 2;
      free (pModData->pBuffer);
      pModData->pBuffer = NULL;
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }
   pModData->pBuffer[iLen-1] = 0;

   pPtr = pModData->pBuffer;
   /*************************************************************************/
   /* Skipp three fields                                                    */
   /*************************************************************************/
   while (*pPtr && (*pPtr != ' '))
      {
      pPtr++;
      }
   if (*pPtr)
      {
      pPtr++;
      }
   while (*pPtr && (*pPtr != ' '))
      {
      pPtr++;
      }

   /*************************************************************************/
   /* Read Flags                                                            */
   /*************************************************************************/
   pModData->ulFlags = 0;
   for (iBase = 0; iBase < 4; iBase++)
      {
      if (*pPtr)
         {
         pPtr++;
         }
      if (*pPtr)
         {
         pPtr++;
         }
      if (*pPtr)
         {
         pPtr++;
         }
      if (*pPtr)
         {
         pModData->ulFlags <<= 4;
         pModData->ulFlags |= (*pPtr <= '9') ?
                               (*pPtr - '0') : ((*pPtr | 0x20) - 'a');
         pPtr++;
         }
      if (*pPtr)
         {
         pModData->ulFlags <<= 4;
         pModData->ulFlags |= (*pPtr <= '9') ?
                               (*pPtr - '0') : ((*pPtr | 0x20) - 'a');
         pPtr++;
         }
      }

   /*************************************************************************/
   /* Read Numbers                                                          */
   /*************************************************************************/
   ReadLong (&pPtr, &pModData->lPercent);  // Read percentage of charge
   if (*pPtr) { pPtr++; }                  // Skipp '%'
   ReadLong (&pPtr, &pModData->lMin);      // Read Minutes

   /*************************************************************************/
   /* Read Units                                                            */
   /*************************************************************************/
   if (*pPtr) { pPtr++; }                  // Skipp ' '
   if (*pPtr == 's')
      {
      pModData->lMin /= 60;
      }

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   char achBuffer [6];
   int iLen;
   long lVal;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;

   iRc = XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                    0, 0,
                    pModData->xpmBGrnd.attributes.width,
                    pModData->xpmBGrnd.attributes.height,
                    iX, iY);

   iLen = 3;
   achBuffer [iLen--] = 0;
   lVal = pModData->lPercent;
   while (iLen >= 0)
      {
      achBuffer [iLen--] = (lVal % 10) + '0';
      lVal /= 10;
      }
   iLen = 0;
   while ((iLen < 2) &&
          (achBuffer [iLen] == '0'))
      {
      achBuffer [iLen++] = ' ';
      }
   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 16, iY + 2, 0);

   if (pModData->lMin >= 0)
      {
      lVal = (pModData->lMin >= 10 * 60) ? 9 : (pModData->lMin / 60);
      achBuffer[0] = (lVal % 10) + '0';
      achBuffer[1] = 0;
      DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 4, iY + 14, 2);

      gcVal.foreground = pModData->pxGreen;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      XDrawLine (pDisplay, hWindow, hGC, iX + 14, iY + 18, iX + 14, iY + 19);
      XDrawLine (pDisplay, hWindow, hGC, iX + 15, iY + 18, iX + 15, iY + 19);
      XDrawLine (pDisplay, hWindow, hGC, iX + 14, iY + 23, iX + 14, iY + 24);
      XDrawLine (pDisplay, hWindow, hGC, iX + 15, iY + 23, iX + 15, iY + 24);

      lVal = (pModData->lMin >= 10 * 60) ? 59 : (pModData->lMin % 60);
      achBuffer[0] = (lVal / 10) + '0';
      achBuffer[1] = (lVal % 10) + '0';
      achBuffer[2] = 0;
      DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 16, iY + 14, 2);
      }

   if (pModData->ulFlags & 0x00010000)
      {
      gcVal.foreground = pModData->pxGreen;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      XDrawLine (pDisplay, hWindow, hGC, iX + 39, iY + 5, iX + 44, iY + 5);
      XDrawLine (pDisplay, hWindow, hGC, iX + 40, iY + 6, iX + 43, iY + 6);
      XDrawLine (pDisplay, hWindow, hGC, iX + 41, iY + 7, iX + 42, iY + 7);
      }
   else
      {
      gcVal.foreground = pModData->pxRed;
      XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
      XDrawLine (pDisplay, hWindow, hGC, iX + 41, iY + 5, iX + 42, iY + 5);
      XDrawLine (pDisplay, hWindow, hGC, iX + 40, iY + 6, iX + 43, iY + 6);
      XDrawLine (pDisplay, hWindow, hGC, iX + 39, iY + 7, iX + 44, iY + 7);
      }

   lVal = pModData->lPercent * 16 / 100;
   if (pModData->lPercent)
      {
      for (iLen = 0; iLen < 6; iLen++)
         {
         gcVal.foreground = pModData->pxBlue[iLen];
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 39 + iLen, iY + 29 - lVal,
                    iX + 39 + iLen, iY + 28);
         }
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.40 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                " apm  V %s\n"
                "\n"
                "%s\n"
                "Time: %li min\n"
                "Battery: %li%%",
                achRev + 11,
                (pModData->ulFlags & 0x00010000) ? "Charging" : "Off Line",
                (pModData->lMin >= 0) ? pModData->lMin : 0,
                pModData->lPercent);
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

