/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: cpu.c,v 1.30 2001/08/21 08:01:33 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: cpu.c,v 1.30 2001/08/21 08:01:33 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataCpu.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define MOD_MAX_COUNT   (10)

#define MOD_CPU_SIZE    (42)
#define MOD_CPU_SIZE_H  (16)

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int      iCount;
   size_t   iBuffSize;
   char *   pBuffer;
   XpmIcon  xpmBGrnd;
   char     achCpu[2];
   char     achPerc[4];
   long     lUserTot;
   long     lNiceTot;
   long     lSysTot;
   long     lIdleTot;
   int      iCurrent;
   long     alCpuTot[MOD_CPU_SIZE];
   long     alCpuSys[MOD_CPU_SIZE];
   long     alCpuNice[MOD_CPU_SIZE];
   long     alCpuUser[MOD_CPU_SIZE];
   Pixel    pxUser;
   Pixel    pxNice;
   Pixel    pxSys;
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;
   char chCpu = ' ';

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options] [cpu_num]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n"
                  "cpu_num:\n"
                  "   0-9          : cpu number\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.30 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else if ((argv[i][0] >= '0') &&
               (argv[i][0] <= '9') &&
               (argv[i][1] == 0))
         {
         chCpu = argv[i][0];
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount     = MOD_MAX_COUNT;
   pModData->iBuffSize  = 1024;
   pModData->pBuffer    = (char *) malloc (pModData->iBuffSize);
   pModData->achCpu[0]  = chCpu;
   pModData->achCpu[1]  = 0;
   pModData->lUserTot   = 0;
   pModData->lNiceTot   = 0;
   pModData->lSysTot    = 0;
   pModData->lIdleTot   = 0;
   pModData->iCurrent   = 0;
   pModData->pxUser     = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->pxNice     = GetColor ("#0000FF", pDisplay, hWindow);
   pModData->pxSys      = GetColor ("#FF0000", pDisplay, hWindow);

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataCpu_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   pModData->achPerc[0]  = ' ';
   pModData->achPerc[1]  = ' ';
   pModData->achPerc[2]  = ' ';
   pModData->achPerc[3]  = 0;

   for (i = 0; i < MOD_CPU_SIZE; i++)
      {
      pModData->alCpuTot[i]  = 0;
      pModData->alCpuUser[i] = 0;
      pModData->alCpuSys[i]  = 0;
      pModData->alCpuNice[i] = 0;
      }

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   pModData->achPerc[0]  = ' ';
   pModData->achPerc[1]  = ' ';
   pModData->achPerc[2]  = ' ';
   pModData->achPerc[3]  = 0;

   pModData->iCurrent     = 0;

   pModData->alCpuTot[0]  = 0;
   pModData->alCpuUser[0] = 0;
   pModData->alCpuSys[0]  = 0;
   pModData->alCpuNice[0] = 0;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   const char * pPtr;
   size_t iLen;
   int hMemFile;
   ModuleData * pModData = (ModuleData *) pData;
   long lNew;
   long lTot;
   long lUser;
   long lNice;
   long lSys;
   long lIdle;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   hMemFile = open ("/proc/stat", O_RDONLY, 0);
   if (hMemFile == -1)
      {
      return (0);
      }

   pModData->pBuffer[0] = '\n';
   iLen = read (hMemFile, pModData->pBuffer + 1, pModData->iBuffSize - 1);
   close (hMemFile);

   if (iLen == pModData->iBuffSize - 1)
      {
      pModData->iBuffSize *= 2;
      free (pModData->pBuffer);
      pModData->pBuffer = NULL;
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }
   pModData->pBuffer[iLen] = 0;

   pPtr = pModData->pBuffer;
   do
      {
      pPtr = strstr (pPtr, "\ncpu");
      }
   while ((pPtr != NULL) && (pPtr[4] != pModData->achCpu[0]) && pPtr++);

   if (pPtr == NULL)
      {
      return (0);
      }

   pPtr += 4;

   /*************************************************************************/
   /* Read Data                                                             */
   /*************************************************************************/
   ReadLong (&pPtr, &lNew);             // Read User
   lUser = lNew - pModData->lUserTot;
   pModData->lUserTot = lNew;

   ReadLong (&pPtr, &lNew);             // Read Nice
   lNice = lNew - pModData->lNiceTot;
   pModData->lNiceTot = lNew;

   ReadLong (&pPtr, &lNew);             // Read Sys
   lSys = lNew - pModData->lSysTot;
   pModData->lSysTot = lNew;

   ReadLong (&pPtr, &lNew);             // Read Idle
   lIdle = lNew - pModData->lIdleTot;
   pModData->lIdleTot = lNew;

   lTot = lUser + lNice + lSys + lIdle;

   /*************************************************************************/
   /* Calc percentages                                                      */
   /*************************************************************************/
   lNew = ((lUser + lNice + lSys) * 100 + 50) / lTot;
   pModData->achPerc[2]  = (lNew % 10) + '0';
   lNew /= 10;
   pModData->achPerc[1]  = (lNew % 10) + '0';
   lNew /= 10;
   pModData->achPerc[0]  = (lNew % 10) + '0';

   if (pModData->achPerc[0] == '0')
      {
      pModData->achPerc[0] = ' ';
      if (pModData->achPerc[1] == '0')
         {
         pModData->achPerc[1] = ' ';
         }
      }

   pModData->alCpuUser[pModData->iCurrent] = lUser;
   pModData->alCpuNice[pModData->iCurrent] = lNice;
   pModData->alCpuSys[pModData->iCurrent]  = lSys;
   pModData->alCpuTot[pModData->iCurrent]  = lUser + lNice + lSys + lIdle;

   pModData->iCurrent = (pModData->iCurrent + 1) % MOD_CPU_SIZE;

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;
   int i, j;
   int iStart, iEnd;
   long lTot;
   long * pTot;
   long * pUser;
   long * pNice;
   long * pSys;

   iRc =  XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                     0, 0,
                     pModData->xpmBGrnd.attributes.width,
                     pModData->xpmBGrnd.attributes.height,
                     iX, iY);

   DrawLetterString (pModData->achCpu, pDisplay, hWindow, hGC,
                     iX + 16, iY + 2);
   DrawDigitString (pModData->achPerc, pDisplay, hWindow, hGC,
                    iX + 26, iY + 2, 0);


   gcVal.foreground = pModData->pxSys;
   XChangeGC (pDisplay, hGC, GCForeground, &gcVal);

   j     = pModData->iCurrent;
   pTot  = &pModData->alCpuTot[j];
   pUser = &pModData->alCpuUser[j];
   pNice = &pModData->alCpuNice[j];
   pSys  = &pModData->alCpuSys[j];
   for (i = 0; i < MOD_CPU_SIZE; i++)
      {
      iEnd = 28;
      lTot = *pTot + 1;

      /**********************************************************************/
      /* Draw Sys Graph                                                     */
      /**********************************************************************/
      iStart = iEnd;
      iEnd   = iStart - MOD_CPU_SIZE_H * (*pSys) / lTot;
      if (*pSys)
         {
         gcVal.foreground = pModData->pxSys;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 3 + i, iY + iStart, iX + 3 + i, iY + iEnd);
         }

      iStart = iEnd;
      iEnd   = iStart - MOD_CPU_SIZE_H * (*pUser) / lTot;
      if (*pUser)
         {
         gcVal.foreground = pModData->pxUser;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 3 + i, iY + iStart, iX + 3 + i, iY + iEnd);
         }

      iStart = iEnd;
      iEnd   = iStart - MOD_CPU_SIZE_H * (*pNice) / lTot;
      if (*pNice)
         {
         gcVal.foreground = pModData->pxNice;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 3 + i, iY + iStart, iX + 3 + i, iY + iEnd);
         }

      if (++j == MOD_CPU_SIZE)
         {
         j     = 0;
         pTot  = &pModData->alCpuTot[0];
         pUser = &pModData->alCpuUser[0];
         pNice = &pModData->alCpuNice[0];
         pSys  = &pModData->alCpuSys[0];
         }
      else
         {
         ++pTot;
         ++pUser;
         ++pNice;
         ++pSys;
         }
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.30 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                " cpu  V %s",
                achRev + 11);
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

