/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: dskstat.c,v 1.25 2001/08/21 08:01:33 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: dskstat.c,v 1.25 2001/08/21 08:01:33 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataDskStat.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>

#define MOD_MAX_COUNT   (10)

#define MOD_DSK_SIZE    (42)
#define MOD_DSK_SIZE_H  (16)                     /* Must be 16 !! */

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int            fScale;
   int            iCount;
   size_t         iBuffSize;
   char *         pBuffer;
   XpmIcon        xpmBGrnd;
   struct timeval tvLast;
   int            iCurrent;
   long           lInTot;
   long           lInMax;
   long           lOutTot;
   long           lOutMax;
   long           alIn[MOD_DSK_SIZE];
   long           alOut[MOD_DSK_SIZE];
   Pixel          pxInColor;
   Pixel          pxOutColor;
   } ModuleData;

static int HiBit (unsigned long lNum);
static void UpdateData22 (const char * pBuffer, long lUSec,
                          long * lRes, long * lTot, long * lMax);
static void UpdateData24 (const char * pBuffer, long lUSec,
                          long * lInRes,  long * lInTot,  long * lInMax,
                          long * lOutRes, long * lOutTot, long * lOutMax);

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;
   int fScale = 1;

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+f") || !strcmp (argv[i], "++fixed"))
         {
         fScale = 0;
         }
      else if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.25 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->fScale     = fScale;
   pModData->iCount     = MOD_MAX_COUNT;
   pModData->iBuffSize  = 1024;
   pModData->pBuffer    = (char *) malloc (pModData->iBuffSize);
   pModData->iCurrent   = 0;
   pModData->pxInColor  = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->pxOutColor = GetColor ("#FF0000", pDisplay, hWindow);

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char**) dataDskStat_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   for (i = 0; i < MOD_DSK_SIZE; i++)
      {
      pModData->alIn[i]  = 0;
      pModData->alOut[i] = 0;
      }
   pModData->lInMax  = 1;
   pModData->lOutMax = 1;

   timerclear (&pModData->tvLast);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   pModData->iCurrent = 0;

   pModData->alIn[0]  = 0;
   pModData->alOut[0] = 0;
   pModData->lInMax   = 1;
   pModData->lOutMax  = 1;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   const char * pPtr;
   size_t iLen;
   int hMemFile;
   ModuleData * pModData = (ModuleData *) pData;
   struct timeval tvNow;
   struct timeval tvDiff;
   long lUSec;
   int fFound;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   gettimeofday(&tvNow, NULL);
   lUSec = 0;
   if ((pModData->tvLast.tv_sec != 0) &&
       (pModData->tvLast.tv_usec != 0))
      {
      timersub (&tvNow, &pModData->tvLast, &tvDiff);
      lUSec = ((tvDiff.tv_sec * 1000000) + tvDiff.tv_usec) / 1000;
      if (lUSec < 0)
         {
         lUSec = -lUSec;
         }
      }
   if (lUSec == 0)
      {
      lUSec = 1;
      }

   hMemFile = open ("/proc/stat", O_RDONLY, 0);
   if (hMemFile == -1)
      {
      return (0);
      }

   pModData->pBuffer[0] = '\n';
   iLen = read (hMemFile, pModData->pBuffer + 1, pModData->iBuffSize - 1);
   close (hMemFile);

   if (iLen == pModData->iBuffSize - 1)
      {
      pModData->iBuffSize *= 2;
      free (pModData->pBuffer);
      pModData->pBuffer = NULL;
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }
   pModData->pBuffer[iLen] = 0;

   /*************************************************************************/
   /* Seach Read Data in 2.2.x kernels                                      */
   /*************************************************************************/
   fFound = 0;
   pPtr = strstr (pModData->pBuffer, "\ndisk_rblk ");
   if (pPtr != NULL)
      {
      fFound = 1;
      pPtr += 11;

      UpdateData22 (pPtr, lUSec,
                    &pModData->alIn[pModData->iCurrent],
                    &pModData->lInTot,
                    &pModData->lInMax);
      }

   /*************************************************************************/
   /* Seach Write Data in 2.2.x kernels                                     */
   /*************************************************************************/
   pPtr = strstr (pModData->pBuffer, "\ndisk_wblk ");
   if (pPtr != NULL)
      {
      fFound = 1;
      pPtr += 11;

      UpdateData22 (pPtr, lUSec,
                    &pModData->alOut[pModData->iCurrent],
                    &pModData->lOutTot,
                    &pModData->lOutMax);
      }

   /*************************************************************************/
   /* Seach Write Data in 2.4.x kernels                                     */
   /*************************************************************************/
   pPtr = strstr (pModData->pBuffer, "\ndisk_io: ");
   if (pPtr != NULL)
      {
      fFound = 1;
      pPtr += 10;

      UpdateData24 (pPtr, lUSec,
                    &pModData->alIn[pModData->iCurrent],
                    &pModData->lInTot,
                    &pModData->lInMax,
                    &pModData->alOut[pModData->iCurrent],
                    &pModData->lOutTot,
                    &pModData->lOutMax);
      }
   if (!fFound)
      {
      return (0);
      }

   pModData->iCurrent = (pModData->iCurrent + 1) % MOD_DSK_SIZE;
   pModData->tvLast = tvNow;

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;
   int i, j;
   int iStart, iEnd;
   long lMax = (pModData->lInMax > pModData->lOutMax) ?
                pModData->lInMax : pModData->lOutMax;

   iRc =  XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                     0, 0,
                     pModData->xpmBGrnd.attributes.width,
                     pModData->xpmBGrnd.attributes.height,
                     iX, iY);

   /*************************************************************************/
   /* Graph                                                                 */
   /*************************************************************************/
   j = pModData->iCurrent;
   for (i = 0; i < MOD_DSK_SIZE; i++)
      {
      iStart = 28;
      iEnd   = iStart -
               ((pModData->fScale) ?
                  (pModData->alIn[j] * (MOD_DSK_SIZE_H - 1) / lMax) :
                  (HiBit ((unsigned long)pModData->alIn[j]) / 2));

      if (pModData->alIn[j])
         {
         gcVal.foreground = pModData->pxInColor;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 3 + i, iY + iStart,
                    iX + 3 + i, iY + iEnd);
         }

      iStart = 13;
      iEnd   = iStart +
               ((pModData->fScale) ?
                  (pModData->alOut[j] * (MOD_DSK_SIZE_H - 1) / lMax) :
                  (HiBit ((unsigned long)pModData->alOut[j]) / 2));
      if (pModData->alOut[j])
         {
         gcVal.foreground = pModData->pxOutColor;
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 3 + i, iY + iStart,
                    iX + 3 + i, iY + iEnd);
         }

      j = (j + 1) % MOD_DSK_SIZE;
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.25 $";

   ModuleData * pModData = (ModuleData *) pData;
   int iCurrent = (pModData->iCurrent + MOD_DSK_SIZE - 1) % MOD_DSK_SIZE;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                "   dskstat  V %s\n"
                "\n"
                "RNow %10li Blk/s\n"
                "RMax %10li Blk/s\n"
                "RTot %10li Blk\n"
                "WNow %10li Blk/s\n"
                "WMax %10li Blk/s\n"
                "WTot %10li Blk",
                achRev + 11,
                pModData->alIn[iCurrent],
                pModData->lInMax,
                pModData->lInTot,
                pModData->alOut[iCurrent],
                pModData->lOutMax,
                pModData->lOutTot);
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

/****************************************************************************/
/*                                HiBit ()                                  */
/****************************************************************************/
static int HiBit (unsigned long lNum)
   {
   int i = 0;
   if (lNum == 0)
      {
      return 0;
      }
   if (lNum & 0xFFFF0000u)
      {
      i += 16;
      lNum >>= 16;
      }
   if (lNum & 0x0000FF00u)
      {
      i += 8;
      lNum >>= 8;
      }
   if (lNum & 0x000000F0u)
      {
      i += 4;
      lNum >>= 4;
      }
   if (lNum & 0x0000000Cu)
      {
      i += 2;
      lNum >>= 2;
      }
   if (lNum & 0x00000002u)
      {
      i += 1;
      lNum >>= 1;
      }
   return (i + (lNum & 0x00000001u));
   }

/****************************************************************************/
/*                            UpdateData22 ()                               */
/****************************************************************************/
static void UpdateData22 (const char * pBuffer, long lUSec,
                          long * lRes, long * lTot, long * lMax)
   {
   long lVal;
   long lNew;
   int i;

   /*************************************************************************/
   /* Read the values                                                       */
   /*************************************************************************/
   lNew = 0;
   for (i = 0; i < 4; i++)
      {
      ReadLong (&pBuffer, &lVal);
      lNew += lVal;
      }

   /*************************************************************************/
   /* Update Read Data                                                      */
   /*************************************************************************/
   lVal = ((lNew - *lTot) * 1000) / lUSec;
   if (lVal < 0)
      {
      lVal = -lVal;
      }
   if ((lNew - *lTot) != 0)
      {
      ++lVal;
      }

   *lRes = lVal;
   *lTot = lNew;
   if (*lMax < lVal)
      {
      *lMax = lVal;
      }

   return;
   }

/****************************************************************************/
/*                            UpdateData24 ()                               */
/****************************************************************************/
static void UpdateData24 (const char * pBuffer, long lUSec,
                          long * lInRes,  long * lInTot, long * lInMax,
                          long * lOutRes, long * lOutTot, long * lOutMax)
   {
   long lInVal, lOutVal;
   long lInNew, lOutNew;

   /*************************************************************************/
   /* Read the values 3 an 5 for each disk                                  */
   /*************************************************************************/
   lInNew = 0;
   lOutNew = 0;
   for (;;)
      {
      while ((*pBuffer != '\n') && (*pBuffer != '('))
         {
         ++pBuffer;
         }
      if (*pBuffer == '(')
         {
         ++pBuffer;
         }
      while ((*pBuffer != '\n') && (*pBuffer != '('))
         {
         ++pBuffer;
         }
      if (*pBuffer == '(')
         {
         ++pBuffer;
         ReadLong (&pBuffer, &lInVal);          // Read drive
         ++pBuffer;
         ReadLong (&pBuffer, &lInVal);          // Read drive_rio
         ++pBuffer;
         ReadLong (&pBuffer, &lInVal);          // Read drive_rblk
         ++pBuffer;
         lInNew += lInVal;
         ReadLong (&pBuffer, &lOutVal);         // Read drive_wio
         ++pBuffer;
         ReadLong (&pBuffer, &lOutVal);         // Read drive_wblk
         ++pBuffer;
         lOutNew += lOutVal;
         }
      else
         {
         break;
         }
      }

   /*************************************************************************/
   /* Update Read Data                                                      */
   /*************************************************************************/
   lInVal = ((lInNew - *lInTot) * 1000) / lUSec;
   if (lInVal < 0)
      {
      lInVal = -lInVal;
      }
   if ((lInNew - *lInTot) != 0)
      {
      ++lInVal;
      }

   *lInRes = lInVal;
   *lInTot = lInNew;
   if (*lInMax < lInVal)
      {
      *lInMax = lInVal;
      }

   lOutVal = ((lOutNew - *lOutTot) * 1000) / lUSec;
   if (lOutVal < 0)
      {
      lOutVal = -lOutVal;
      }
   if ((lOutNew - *lOutTot) != 0)
      {
      ++lOutVal;
      }

   *lOutRes = lOutVal;
   *lOutTot = lOutNew;
   if (*lOutMax < lOutVal)
      {
      *lOutMax = lOutVal;
      }
   return;
   }

