/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: netstat.c,v 1.27 2001/08/21 08:01:33 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: netstat.c,v 1.27 2001/08/21 08:01:33 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataNetStat.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>

#define MOD_MAX_COUNT   (10)

#define MOD_NET_SIZE    (42)
#define MOD_NET_SIZE_H  (16)                     /* Must be 16 !! */

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int            fScale;
   int            iCount;
   size_t         iBuffSize;
   char *         pBuffer;
   XpmIcon        xpmBGrnd;
   struct timeval tvLast;
   char           achNet[16];
   int            iCurrent;
   long           lInTot[8];
   long           lInMax[8];
   long           lOutTot[8];
   long           lOutMax[8];
   long           alIn[MOD_NET_SIZE][8];
   long           alOut[MOD_NET_SIZE][8];
   Pixel          pxColor[16];
   } ModuleData;

static int HiBit (unsigned long lNum);

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i, j;
   const char * pNet = "lo";
   int fScale = 1;

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options] [device]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n"
                  "   +f ++fixed   : don't use variable scale\n"
                  "device:\n"
                  "   lo eth1 tr0  : device name\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.27 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+f") || !strcmp (argv[i], "++fixed"))
         {
         fScale = 0;
         }
      else if ((argv[i][0] != '-') && (argv[i][0] != '+'))
         {
         pNet = argv[i];
         if (strlen (pNet) > 14)
            {
            fprintf (stderr, "%s: parameter %s too long\n", argv[0], argv[i]);
            iRc = 1;
            }
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->fScale      = fScale;
   pModData->iCount      = MOD_MAX_COUNT;
   pModData->iBuffSize   = 2048;
   pModData->pBuffer     = (char *) malloc (pModData->iBuffSize);
   pModData->iCurrent    = 0;
   pModData->pxColor[0]  = GetColor ("#EEFF00", pDisplay, hWindow);
   pModData->pxColor[1]  = GetColor ("#CCFF00", pDisplay, hWindow);
   pModData->pxColor[2]  = GetColor ("#AAFF00", pDisplay, hWindow);
   pModData->pxColor[3]  = GetColor ("#88FF00", pDisplay, hWindow);
   pModData->pxColor[4]  = GetColor ("#66FF00", pDisplay, hWindow);
   pModData->pxColor[5]  = GetColor ("#44FF00", pDisplay, hWindow);
   pModData->pxColor[6]  = GetColor ("#22FF00", pDisplay, hWindow);
   pModData->pxColor[7]  = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->pxColor[8]  = GetColor ("#00FF00", pDisplay, hWindow);
   pModData->pxColor[9]  = GetColor ("#00FF22", pDisplay, hWindow);
   pModData->pxColor[10] = GetColor ("#00FF44", pDisplay, hWindow);
   pModData->pxColor[11] = GetColor ("#00FF66", pDisplay, hWindow);
   pModData->pxColor[12] = GetColor ("#00FF88", pDisplay, hWindow);
   pModData->pxColor[13] = GetColor ("#00FFAA", pDisplay, hWindow);
   pModData->pxColor[14] = GetColor ("#00FFCC", pDisplay, hWindow);
   pModData->pxColor[15] = GetColor ("#00FFEE", pDisplay, hWindow);
   strcpy (pModData->achNet, pNet);
   strcat (pModData->achNet, ":");

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataNetStat_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   for (j = 0; j < 8; j++)
      {
      pModData->lInTot[j]  = 0;
      pModData->lOutTot[j] = 0;
      pModData->lInMax[j] = 1;
      pModData->lOutMax[j] = 1;
      for (i = 0; i < MOD_NET_SIZE; i++)
         {
         pModData->alIn[i][j]  = 0;
         pModData->alOut[i][j] = 0;
         }
      }

   timerclear (&pModData->tvLast);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   pModData->iCurrent = 0;
   for (j = 0; j < 8; j++)
      {
      pModData->alIn[0][j]  = 0;
      pModData->alOut[0][j] = 0;
      pModData->lInMax[j] = 1;
      pModData->lOutMax[j] = 1;
      }

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   const char * pPtr;
   size_t iLen;
   int hMemFile;
   ModuleData * pModData = (ModuleData *) pData;
   int i;
   long alNewVal[16];
   struct timeval tvNow;
   struct timeval tvDiff;
   long lUSec;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   gettimeofday(&tvNow, NULL);
   lUSec = 0;
   if ((pModData->tvLast.tv_sec != 0) &&
       (pModData->tvLast.tv_usec != 0))
      {
      timersub (&tvNow, &pModData->tvLast, &tvDiff);
      lUSec = ((tvDiff.tv_sec * 1000000) + tvDiff.tv_usec) / 1000;
      if (lUSec < 0)
         {
         lUSec = -lUSec;
         }
      }
   if (lUSec == 0)
      {
      lUSec = 1;
      }

   hMemFile = open ("/proc/net/dev", O_RDONLY, 0);
   if (hMemFile == -1)
      {
      return (0);
      }

   pModData->pBuffer[0] = ' ';
   iLen = read (hMemFile, pModData->pBuffer + 1, pModData->iBuffSize - 1);
   close (hMemFile);

   if (iLen == pModData->iBuffSize - 1)
      {
      pModData->iBuffSize *= 2;
      free (pModData->pBuffer);
      pModData->pBuffer = NULL;
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }
   pModData->pBuffer[iLen] = 0;

   /*************************************************************************/
   /* Seach Interface                                                       */
   /*************************************************************************/
   pPtr = strstr (pModData->pBuffer, pModData->achNet);
   if (pPtr == NULL)
      {
      return (0);
      }

   while (*pPtr && (*pPtr != ':'))
      {
      pPtr++;
      }
   if (*pPtr)
      {
      pPtr++;
      }

   /*************************************************************************/
   /* Read the values                                                       */
   /*************************************************************************/
   for (i = 0; i < 16; i++)
      {
      ReadLong (&pPtr, &alNewVal[i]);
      }
   for (i = 0; i < 8; i++)
      {
      pModData->alIn[pModData->iCurrent][i] =
         ((alNewVal[i] - pModData->lInTot[i]) * 1000) / lUSec;
      if (pModData->alIn[pModData->iCurrent][i] < 0)
         {
         pModData->alIn[pModData->iCurrent][i] =
            -pModData->alIn[pModData->iCurrent][i];
         }
      if ((alNewVal[i] - pModData->lInTot[i]) != 0)
         {
         ++(pModData->alIn[pModData->iCurrent][i]);
         }
      pModData->lInTot[i] = alNewVal[i];
      if (pModData->lInMax[i] < pModData->alIn[pModData->iCurrent][i])
         {
         pModData->lInMax[i] = pModData->alIn[pModData->iCurrent][i];
         }

      pModData->alOut[pModData->iCurrent][i] =
         ((alNewVal[i+8] - pModData->lOutTot[i]) * 1000) / lUSec;
      if (pModData->alOut[pModData->iCurrent][i] < 0)
         {
         pModData->alOut[pModData->iCurrent][i] =
            -pModData->alOut[pModData->iCurrent][i];
         }
      if ((alNewVal[i+8] - pModData->lOutTot[i]) != 0)
         {
         ++(pModData->alOut[pModData->iCurrent][i]);
         }
      pModData->lOutTot[i] = alNewVal[i+8];

      if (pModData->lOutMax[i] < pModData->alOut[pModData->iCurrent][i])
         {
         pModData->lOutMax[i] = pModData->alOut[pModData->iCurrent][i];
         }
      }

   pModData->iCurrent = (pModData->iCurrent + 1) % MOD_NET_SIZE;
   pModData->tvLast = tvNow;

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;
   int i, j, k;
   int iStartTop,  iEndTop;
   int iStartDown, iEndDown;
   long lMax;

   iRc =  XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                     0, 0,
                     pModData->xpmBGrnd.attributes.width,
                     pModData->xpmBGrnd.attributes.height,
                     iX, iY);

   i = strlen (pModData->achNet);
   pModData->achNet[i-1] = 0;
   DrawLetterString (pModData->achNet, pDisplay, hWindow, hGC,
                     iX + 2, iY + 2);
   pModData->achNet[i-1] = ':';


   /*************************************************************************/
   /* Graph                                                                 */
   /*************************************************************************/
   j = pModData->iCurrent;
   for (i = 0; i < MOD_NET_SIZE; i++)
      {
      iEndDown = 28;
      iEndTop  = 13;

      for (k = 0; k < 1; k++)           /* Do only bytes */
         {
         lMax = (pModData->lInMax[k] > pModData->lOutMax[k]) ?
                 pModData->lInMax[k] : pModData->lOutMax[k];

         iStartDown = iEndDown;
         iEndDown   = iStartDown -
                      ((pModData->fScale) ?
                         (pModData->alIn[j][k] * (MOD_NET_SIZE_H - 1) / lMax) :
                         (HiBit ((unsigned long)pModData->alIn[j][k]) / 2));

         if (pModData->alIn[j][k])
            {
            gcVal.foreground = pModData->pxColor[k];
            XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
            XDrawLine (pDisplay, hWindow, hGC,
                       iX + 3 + i, iY + iStartDown,
                       iX + 3 + i, iY + iEndDown);
            }

         iStartTop = iEndTop;
         iEndTop   = iStartTop +
                     ((pModData->fScale) ?
                        (pModData->alOut[j][k] * (MOD_NET_SIZE_H - 1) / lMax):
                        (HiBit ((unsigned long)pModData->alOut[j][k]) / 2));

         if (pModData->alOut[j][k])
            {
            gcVal.foreground = pModData->pxColor[k+8];
            XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
            XDrawLine (pDisplay, hWindow, hGC,
                       iX + 3 + i, iY + iStartTop,
                       iX + 3 + i, iY + iEndTop);
            }
         }

      j = (j + 1) % MOD_NET_SIZE;
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.27 $";
   ModuleData * pModData = (ModuleData *) pData;
   int iCurrent = (pModData->iCurrent + MOD_NET_SIZE - 1) % MOD_NET_SIZE;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                "   netstat  V %s\n"
                "\n"
                "RNow %10li B/s\n"
                "RMax %10li B/s\n"
                "RTot %10li Bytes\n"
                "WNow %10li B/s\n"
                "WMax %10li B/s\n"
                "WTot %10li Bytes",
                achRev + 11,
                pModData->alIn[iCurrent][0],
                pModData->lInMax[0],
                pModData->lInTot[0],
                pModData->alOut[iCurrent][0],
                pModData->lOutMax[0],
                pModData->lOutTot[0]);
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                HiBit ()                                  */
/****************************************************************************/
static int HiBit (unsigned long lNum)
   {
   int i = 0;
   if (lNum == 0)
      {
      return 0;
      }
   if (lNum & 0xFFFF0000u)
      {
      i += 16;
      lNum >>= 16;
      }
   if (lNum & 0x0000FF00u)
      {
      i += 8;
      lNum >>= 8;
      }
   if (lNum & 0x000000F0u)
      {
      i += 4;
      lNum >>= 4;
      }
   if (lNum & 0x0000000Cu)
      {
      i += 2;
      lNum >>= 2;
      }
   if (lNum & 0x00000002u)
      {
      i += 1;
      lNum >>= 1;
      }
   return (i + (lNum & 0x00000001u));
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

