/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: serial.c,v 1.32 2001/08/21 08:01:34 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: serial.c,v 1.32 2001/08/21 08:01:34 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataSerial.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <string.h>

#define MOD_MAX_COUNT   (2)

#define FLAG_RTS        (0x01)
#define FLAG_CTS        (0x02)
#define FLAG_DTR        (0x04)
#define FLAG_DSR        (0x08)
#define FLAG_CD         (0x10)
#define FLAG_RI         (0x20)
#define FLAG_TX         (0x40)
#define FLAG_RX         (0x80)

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int      iCount;
   size_t   iBuffSize;
   char *   pBuffer;
   XpmIcon  xpmBGrnd;
   char     achDriver[14];
   long     lTx;
   long     lRx;
   char     achSerial[3];
   char   * pcszFile;
   unsigned uiFlags;
   Pixel    pxRed;
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;
   char chSerial = '0';
   char chSerial1 = 0 ;
   const char * pcszFile = "/proc/tty/driver/serial";

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options] [nn]\n"
                  "options:\n"
                  "   +h ++help          : print this help\n"
                  "   +s ++serial <file> : serial proc file\n"
                  "                           default is "
                                       "\"/proc/tty/driver/serial\"\n"
                  "   +V ++version       : print version\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+s") || !strcmp (argv[i], "++serial"))
         {
         ++i;
         if (i == argc)
            {
            fprintf (stderr, "%s: serial file is required\n", argv[0]);
            return (NULL);
            }
         pcszFile = argv[i];
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.32 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else if ((argv[i][0] >= '0') && (argv[i][0] <= '9') &&
               ((argv[i][1] == 0) ||
                ((argv[i][1] >= '0') && (argv[i][1] <= '9') &&
                 (argv[i][2] == 0))))
         {
         chSerial  = argv[i][0];
         chSerial1 = argv[i][1];
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      fprintf (stderr, "%s: out of memory\n", argv[0]);
      return (NULL);
      }

   pModData->iCount       = MOD_MAX_COUNT;
   pModData->iBuffSize    = 256;
   pModData->pBuffer      = (char *) malloc (pModData->iBuffSize);
   pModData->achDriver[0] = 0;
   pModData->uiFlags      = 0;
   pModData->lTx          = 0;
   pModData->lRx          = 0;
   pModData->pxRed        = GetColor ("#FF0000", pDisplay, hWindow);
   pModData->achSerial[0] = chSerial;
   pModData->achSerial[1] = chSerial1;
   pModData->achSerial[2] = 0;
   pModData->pcszFile     = strdup (pcszFile);

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataSerial_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   const char * pPtr;
   const char * pStart;
   char * pStr;
   size_t iLen;
   int hMemFile;
   ModuleData * pModData = (ModuleData *) pData;
   long lNew;
   char achSer[5];

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   pModData->uiFlags = 0;

   hMemFile = open (pModData->pcszFile, O_RDONLY, 0);
   if (hMemFile == -1)
      {
      return (0);
      }

   pModData->pBuffer[0] = '\n';
   iLen = read (hMemFile, pModData->pBuffer + 1, pModData->iBuffSize - 1);
   close (hMemFile);

   if (iLen == pModData->iBuffSize - 1)
      {
      pModData->iBuffSize *= 2;
      free (pModData->pBuffer);
      pModData->pBuffer = NULL;
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }
   pModData->pBuffer[iLen] = 0;

   /*************************************************************************/
   /* Search Serial                                                         */
   /*************************************************************************/
   achSer[0] = '\n';
   achSer[1] = pModData->achSerial[0];
   achSer[2] = pModData->achSerial[1] ? pModData->achSerial[1] : ':';
   achSer[3] = pModData->achSerial[1] ? ':' : 0;
   achSer[4] = 0;

   pPtr = strstr (pModData->pBuffer, achSer);

   if ((pPtr == NULL ) ||
       (*pPtr == 0))
      {
      return 0;
      }
   pStart = ++pPtr;

   while ((*pPtr != 0) && (*pPtr != '\n'))
      {
      pPtr++;
      }
   *(char*)pPtr = 0;

   if (strstr (pStart, "RTS"))
      {
      pModData->uiFlags |= FLAG_RTS;
      }
   if (strstr (pStart, "CTS"))
      {
      pModData->uiFlags |= FLAG_CTS;
      }
   if (strstr (pStart, "DTR"))
      {
      pModData->uiFlags |= FLAG_DTR;
      }
   if (strstr (pStart, "DSR"))
      {
      pModData->uiFlags |= FLAG_DSR;
      }
   if (strstr (pStart, "CD"))
      {
      pModData->uiFlags |= FLAG_CD;
      }
   if (strstr (pStart, "RI"))
      {
      pModData->uiFlags |= FLAG_RI;
      }

   pPtr = strstr (pStart, "tx:");
   if (pPtr)
      {
      pPtr += 3;
      ReadLong (&pPtr, &lNew);

      if (lNew != pModData->lTx)
         {
         pModData->lTx = lNew;
         pModData->uiFlags |= FLAG_TX;
         }
      }

   pPtr = strstr (pStart, "rx:");
   if (pPtr)
      {
      pPtr += 3;
      ReadLong (&pPtr, &lNew);

      if (lNew != pModData->lRx)
         {
         pModData->lRx = lNew;
         pModData->uiFlags |= FLAG_RX;
         }
      }

   pPtr = strstr (pStart, "uart:");
   if (pPtr)
      {
      pPtr += 5;
      pStr = pModData->achDriver;
      iLen = 1;

      while (*pPtr && (*pPtr != ' ') &&
             (iLen < sizeof(pModData->achDriver)))
         {
         *pStr++ = *pPtr++;
         ++iLen;
         }
      *pStr = 0;
      }

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;

   iRc =  XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                     0, 0,
                     pModData->xpmBGrnd.attributes.width,
                     pModData->xpmBGrnd.attributes.height,
                     iX, iY);

   DrawLetterString (pModData->achSerial, pDisplay, hWindow, hGC,
                     iX + 34, iY + 1);

   gcVal.foreground = pModData->pxRed;
   XChangeGC (pDisplay, hGC, GCForeground, &gcVal);

   if (pModData->uiFlags & FLAG_RX)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 14, iY + 12, iX + 21, iY + 12);
      XDrawLine (pDisplay, hWindow, hGC, iX + 14, iY + 13, iX + 21, iY + 13);
      XDrawLine (pDisplay, hWindow, hGC, iX + 14, iY + 14, iX + 21, iY + 14);
      XDrawLine (pDisplay, hWindow, hGC, iX + 14, iY + 15, iX + 21, iY + 15);
      }
   if (pModData->uiFlags & FLAG_TX)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 36, iY + 12, iX + 43, iY + 12);
      XDrawLine (pDisplay, hWindow, hGC, iX + 36, iY + 13, iX + 43, iY + 13);
      XDrawLine (pDisplay, hWindow, hGC, iX + 36, iY + 14, iX + 43, iY + 14);
      XDrawLine (pDisplay, hWindow, hGC, iX + 36, iY + 15, iX + 43, iY + 15);
      }

   if (pModData->uiFlags & FLAG_RTS)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 4, iY + 25, iX + 7, iY + 25);
      XDrawLine (pDisplay, hWindow, hGC, iX + 4, iY + 26, iX + 7, iY + 26);
      XDrawLine (pDisplay, hWindow, hGC, iX + 4, iY + 27, iX + 7, iY + 27);
      XDrawLine (pDisplay, hWindow, hGC, iX + 4, iY + 28, iX + 7, iY + 28);
      }
   if (pModData->uiFlags & FLAG_CTS)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 11, iY + 25, iX + 14, iY + 25);
      XDrawLine (pDisplay, hWindow, hGC, iX + 11, iY + 26, iX + 14, iY + 26);
      XDrawLine (pDisplay, hWindow, hGC, iX + 11, iY + 27, iX + 14, iY + 27);
      XDrawLine (pDisplay, hWindow, hGC, iX + 11, iY + 28, iX + 14, iY + 28);
      }
   if (pModData->uiFlags & FLAG_DTR)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 18, iY + 25, iX + 21, iY + 25);
      XDrawLine (pDisplay, hWindow, hGC, iX + 18, iY + 26, iX + 21, iY + 26);
      XDrawLine (pDisplay, hWindow, hGC, iX + 18, iY + 27, iX + 21, iY + 27);
      XDrawLine (pDisplay, hWindow, hGC, iX + 18, iY + 28, iX + 21, iY + 28);
      }

   if (pModData->uiFlags & FLAG_DSR)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 26, iY + 25, iX + 29, iY + 25);
      XDrawLine (pDisplay, hWindow, hGC, iX + 26, iY + 26, iX + 29, iY + 26);
      XDrawLine (pDisplay, hWindow, hGC, iX + 26, iY + 27, iX + 29, iY + 27);
      XDrawLine (pDisplay, hWindow, hGC, iX + 26, iY + 28, iX + 29, iY + 28);
      }
   if (pModData->uiFlags & FLAG_CD)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 33, iY + 25, iX + 36, iY + 25);
      XDrawLine (pDisplay, hWindow, hGC, iX + 33, iY + 26, iX + 36, iY + 26);
      XDrawLine (pDisplay, hWindow, hGC, iX + 33, iY + 27, iX + 36, iY + 27);
      XDrawLine (pDisplay, hWindow, hGC, iX + 33, iY + 28, iX + 36, iY + 28);
      }
   if (pModData->uiFlags & FLAG_RI)
      {
      XDrawLine (pDisplay, hWindow, hGC, iX + 40, iY + 25, iX + 43, iY + 25);
      XDrawLine (pDisplay, hWindow, hGC, iX + 40, iY + 26, iX + 43, iY + 26);
      XDrawLine (pDisplay, hWindow, hGC, iX + 40, iY + 27, iX + 43, iY + 27);
      XDrawLine (pDisplay, hWindow, hGC, iX + 40, iY + 28, iX + 43, iY + 28);
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.32 $";

   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                "   serial  V %s\n"
                "\n"
                "Driver %s\n"
                "RTot %10li Bytes\n"
                "WTot %10li Bytes\n"
                "Flags  %s %s %s %s\n"
                "       %s %s %s %s",
                achRev + 11,
                pModData->achDriver,
                pModData->lRx,
                pModData->lTx,
                (pModData->uiFlags & FLAG_RTS) ? "RTS" : "---",
                (pModData->uiFlags & FLAG_CTS) ? "CTS" : "---",
                (pModData->uiFlags & FLAG_CD ) ? "CD" : "--",
                (pModData->uiFlags & FLAG_RI ) ? "RI" : "--",
                (pModData->uiFlags & FLAG_DTR) ? "DTR" : "---",
                (pModData->uiFlags & FLAG_DSR) ? "DSR" : "---",
                (pModData->uiFlags & FLAG_TX ) ? "TX" : "--",
                (pModData->uiFlags & FLAG_RX ) ? "RX" : "--");
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

