/****************************************************************************/
/* |\   ======   /| **              MSysM               ** |\   ======   /| */
/* |__\========/__| **     MasarLabs System Monitor     ** |__\========/__| */
/* |\            /| ** -------------------------------- ** |\            /| */
/* |  \        /  | **  Maurizio Sartori (c) 1998-2000  ** |  \        /  | */
/* |    \    /    | ** -------------------------------- ** |    \    /    | */
/* |    /=\/=\    | **            Written  by           ** |    /=\/=\    | */
/* |  /========\  | **     Maurizio Sartori 'masar'     ** |  /========\  | */
/* |/   ======   \| **   e-mail:  masar@MasarLabs.com   ** |/   ======   \| */
/****************************************************************************/
/*              This program comes with ABSOLUTELY NO WARRANTY              */
/*            for details refer to the GNU General Public License           */
/****************************************************************************/
/*\F $Id: swap.c,v 1.35 2001/08/21 08:01:34 cvs Exp $
*****************************************************************************/

static char Psz_RCS [] =
    "$Id: swap.c,v 1.35 2001/08/21 08:01:34 cvs Exp $";

/****************************************************************************/

#include <msysm/modules.h>
#include "dataSwap.xpm"

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#define MOD_MAX_COUNT   (10)

/****************************************************************************/
/* Local Types                                                              */
/****************************************************************************/
typedef struct _ModuleData
   {
   int      iCount;
   size_t   iBuffSize;
   char *   pBuffer;
   XpmIcon  xpmBGrnd;
   long     lTotSwap;
   long     lUsedSwap;
   Pixel    pxUsed[6];
   } ModuleData;

/****************************************************************************/
/*                                  Load ()                                 */
/****************************************************************************/
void * Load (int argc, const char * argv [],
             Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData;
   int iRc;
   int i;

   *Psz_RCS = *Psz_RCS;                         // To fake compiler warnings

   iRc = 0;
   i = 1;
   while (i < argc)
      {
      if (!strcmp (argv[i], "+h") || !strcmp (argv[i], "++help"))
         {
         fprintf (stdout,
                  "usage: %s [options]\n"
                  "options:\n"
                  "   +h ++help    : print this help\n"
                  "   +V ++version : print version\n",
                  argv[0]);
         iRc = 1;
         }
      else if (!strcmp (argv[i], "+V") || !strcmp (argv[i], "++version"))
         {
         char achRev [] = "$Revision: 1.35 $";
         achRev[strlen(achRev)-1] = '\0';

         fprintf (stdout, "%-10s Ver "
                  APP_STR_MAJOR "." APP_STR_MINOR "." APP_STR_REVIS
                  " Rev %s\n", argv[0], achRev + 11);
         iRc = 1;
         }
      else
         {
         fprintf (stderr, "%s: invalid parameter %s\n", argv[0], argv[i]);
         iRc = 1;
         }
      i++;
      }

   if (iRc)
      {
      return (NULL);
      }

   pModData = (ModuleData *) malloc (sizeof (ModuleData));
   if (pModData == NULL)
      {
      return (NULL);
      }

   pModData->iCount     = rand () % MOD_MAX_COUNT;
   pModData->iBuffSize  = 256;
   pModData->pBuffer    = (char *) malloc (pModData->iBuffSize);
   pModData->lTotSwap   = 0;
   pModData->lUsedSwap  = 0;
   pModData->pxUsed[0]  = GetColor ("#38FFFF", pDisplay, hWindow);
   pModData->pxUsed[1]  = GetColor ("#34EEEE", pDisplay, hWindow);
   pModData->pxUsed[2]  = GetColor ("#30DDDD", pDisplay, hWindow);
   pModData->pxUsed[3]  = GetColor ("#28CCCC", pDisplay, hWindow);
   pModData->pxUsed[4]  = GetColor ("#24BBBB", pDisplay, hWindow);
   pModData->pxUsed[5]  = GetColor ("#20AAAA", pDisplay, hWindow);

   memset (&pModData->xpmBGrnd, 0, sizeof (pModData->xpmBGrnd));
   pModData->xpmBGrnd.attributes.valuemask = (XpmReturnPixels |
                                              XpmReturnExtensions);

   XpmCreatePixmapFromData (pDisplay, hWindow,
                            (char **) dataSwap_xpm,
                            &pModData->xpmBGrnd.pixmap,
                            &pModData->xpmBGrnd.mask,
                            &pModData->xpmBGrnd.attributes);

   Update (pModData);
   pModData->iCount = rand () % MOD_MAX_COUNT;

   return (pModData);
   }

/****************************************************************************/
/*                                 UnLoad ()                                */
/****************************************************************************/
void UnLoad (void * pData, Display * pDisplay, Window hWindow)
   {
   ModuleData * pModData = (ModuleData *) pData;
   (void) hWindow;

   XFreePixmap (pDisplay, pModData->xpmBGrnd.pixmap);
   free (pModData->pBuffer);
   free (pModData);
   }

/****************************************************************************/
/*                                 Update ()                                */
/****************************************************************************/
int Update (void * pData)
   {
   const char * pPtr;
   size_t iLen;
   int hMemFile;
   ModuleData * pModData = (ModuleData *) pData;
   long lNew;
   int iBase;

   pModData->iCount++;
   if (pModData->iCount < MOD_MAX_COUNT)
      {
      return (0);
      }
   pModData->iCount = 0;

   if (pModData->pBuffer == NULL)
      {
      pModData->pBuffer = (char *) malloc (pModData->iBuffSize);
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }

   hMemFile = open ("/proc/meminfo", O_RDONLY, 0);
   if (hMemFile == -1)
      {
      return (0);
      }

   iLen = read (hMemFile, pModData->pBuffer, pModData->iBuffSize);
   close (hMemFile);

   if (iLen == pModData->iBuffSize)
      {
      pModData->iBuffSize *= 2;
      free (pModData->pBuffer);
      pModData->pBuffer = NULL;
      pModData->iCount = MOD_MAX_COUNT;
      return (0);
      }
   pModData->pBuffer[iLen-1] = 0;

   /*************************************************************************/
   /* Read Data                                                             */
   /*************************************************************************/
   pPtr = strstr (pModData->pBuffer, "\nSwap:");
   if (pPtr == NULL)
      {
      return (0);
      }
   pPtr += 6;

   ReadLong (&pPtr, &lNew);             // Read Total
   iBase = 1;
   while (lNew > 999999)
      {
      lNew /= 1024;
      iBase += 1024;
      }
   pModData->lTotSwap = lNew;

   ReadLong (&pPtr, &lNew);             // Read Used
   pModData->lUsedSwap = lNew / iBase;

   return (1);
   }

/****************************************************************************/
/*                                 Paint ()                                 */
/****************************************************************************/
void Paint (Display * pDisplay, Window hWindow,
            GC        hGC, int iX, int iY,
            void    * pData)
   {
   int iRc;
   char achBuffer [6];
   int iLen;
   long lVal;
   ModuleData * pModData = (ModuleData *) pData;
   XGCValues gcVal;

   iRc =  XCopyArea (pDisplay, pModData->xpmBGrnd.pixmap, hWindow, hGC,
                     0, 0,
                     pModData->xpmBGrnd.attributes.width,
                     pModData->xpmBGrnd.attributes.height,
                     iX, iY);

   iLen = 3;
   achBuffer [iLen--] = 0;
   lVal = pModData->lUsedSwap * 100 /
          ((pModData->lTotSwap) ? pModData->lTotSwap : 1);
   while (iLen >= 0)
      {
      achBuffer [iLen--] = (lVal % 10) + '0';
      lVal /= 10;
      }
   iLen = 0;
   while ((iLen < 2) &&
          (achBuffer [iLen] == '0'))
      {
      achBuffer [iLen++] = ' ';
      }
   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 16, iY + 2, 0);

   iLen = sizeof (achBuffer);
   achBuffer [iLen--] = 0;
   lVal = pModData->lTotSwap;
   while (iLen >= 0)
      {
      achBuffer [iLen--] = (lVal % 10) + '0';
      lVal /= 10;
      }
   iLen = 0;
   while (((size_t) iLen < (sizeof (achBuffer) - 1)) &&
          (achBuffer [iLen] == '0'))
      {
      achBuffer [iLen++] = ' ';
      }
   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 6, iY + 12, 0);

   iLen = sizeof (achBuffer);
   achBuffer [iLen--] = 0;
   lVal = pModData->lUsedSwap;
   while (iLen >= 0)
      {
      achBuffer [iLen--] = (lVal % 10) + '0';
      lVal /= 10;
      }
   iLen = 0;
   while (((size_t) iLen < (sizeof (achBuffer) - 1)) &&
          (achBuffer [iLen] == '0'))
      {
      achBuffer [iLen++] = ' ';
      }
   DrawDigitString (achBuffer, pDisplay, hWindow, hGC, iX + 6, iY + 22, 0);

   lVal = pModData->lUsedSwap * 24 /
          ((pModData->lTotSwap) ? pModData->lTotSwap : 1);
   if (pModData->lUsedSwap)
      {
      for (iLen = 0; iLen < 6; iLen++)
         {
         gcVal.foreground = pModData->pxUsed[iLen];
         XChangeGC (pDisplay, hGC, GCForeground, &gcVal);
         XDrawLine (pDisplay, hWindow, hGC,
                    iX + 39 + iLen, iY + 28 - lVal,
                    iX + 39 + iLen, iY + 28);
         }
      }
   }

/****************************************************************************/
/*                                 Info ()                                  */
/****************************************************************************/
const char * Info (void * pData)
   {
   static char achRev [] = "$Revision: 1.35 $";
   ModuleData * pModData = (ModuleData *) pData;

   if (achRev[strlen (achRev)-1] == '$')
      {
      achRev[strlen (achRev)-1] = 0;
      }

   if (pModData->pBuffer != NULL)
      {
      snprintf (pModData->pBuffer,
                pModData->iBuffSize,
                " swap  V %s",
                achRev + 11);
      }

   return (pModData->pBuffer);
   }

/****************************************************************************/
/*                                 Exec ()                                  */
/****************************************************************************/
void Exec (void * pData)
   {
   ModuleData * pModData = (ModuleData *) pData;

   (void) pModData;             /* Do Nothing !!                            */

   return;
   }

